## Script (Python) "xhostplusListAlbum"
##title=Helper method for xhostplus gallery view
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=images=0, folders=0, subimages=0

from Products.CMFPlone.utils import base_hasattr

result = {}

if images:
    result['images'] = context.getFolderContents({
        'portal_type': ('Image',)
    }, full_objects=True)
if folders:
    # We don't need the full objects for the folders
    result['folders'] = context.getFolderContents({
        'portal_type':('Image Gallery',)
    })
if subimages:
    #Handle brains or objects
    if base_hasattr(context, 'getPath'):
        path = context.getPath()
    else:
        path = '/'.join(context.getPhysicalPath())
    # Explicitly set path to remove default depth
    result['subimages'] = context.getFolderContents({
        'Type':('Image',), 
        'path':path
    })

return result



import mimetypes
import urllib

from zope.interface import implements, Interface
from interfaces import IFileMutator

from Acquisition import aq_inner

from zope import component
from zope.filerepresentation.interfaces import IFileFactory

from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from xhostplus.gallery import galleryMessageFactory as _

try :
    #python >=2.6
    import json
except :
    #plone 3
    import simplejson as json

XHR_UPLOAD_JS = """
    var uploader = new qq.FileUploader({
        element: jq('#file_uploader')[0],
        action: '@@add_image',
        allowedExtensions: ['jpg', 'jpeg', 'png'],
        template: '<div class="qq-uploader">' +
                  '<div class="qq-upload-drop-area"><span>%(ul_draganddrop_text)s</span></div>' +
                  '<div class="qq-upload-button">%(ul_button_text)s</div>' +
                  '<ul class="qq-upload-list"></ul>' + 
                  '</div>',
        fileTemplate: '<li>' +
                '<div class="qq-upload-infos"><span class="qq-upload-file"></span>' +
                '<span class="qq-upload-spinner"></span>' +
                '<span class="qq-upload-failed-text">%(ul_msg_failed)s</span></div>' +
                '<div class="qq-upload-size"></div>' +
                '<a class="qq-upload-cancel" href="#">%(ul_button_cancel)s</a>' +
            '</li>',
        messages: {
            serverError: "%(ul_error_server)s",
            alreadyExistsError: "%(ul_error_already_exists)s {file}",
            typeError: "%(ul_error_bad_ext)s {file}. %(ul_error_onlyallowed)s {extensions}.",
            sizeError: "%(ul_error_file_large)s {file}, %(ul_error_maxsize_is)s {sizeLimit}.</span>",
            emptyError: "%(ul_error_empty_file)s {file}, %(ul_error_try_again)s"
        }
    });
"""

def encode(s):
    """
    encode string
    """
    return "d".join(map(str, map(ord, s)))


def decode(s):
    """
    decode string
    """
    return "".join(map(chr, map(int, s.split("d"))))

class IAddImagesView(Interface):
    """
    add-images view interface
    """


class AddImagesView(BrowserView):
    """
    add-images browser view
    """
    implements(IAddImagesView)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def portal_catalog(self):
        return getToolByName(self.context, 'portal_catalog')

    @property
    def portal(self):
        return getToolByName(self.context, 'portal_url').getPortalObject()

class AddImagesJs(BrowserView):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def _utranslate(self, msg):
        context = aq_inner(self.context)
        return context.translate(msg, domain="xhostplus.gallery")

    def __call__(self):
        _ = self._utranslate
        strings = dict(
            ul_button_text          = _(u'Add one or more images'),
            ul_button_cancel        = _('Cancel'),
            ul_draganddrop_text     = _(u'Drag and drop images here to add'),
            ul_msg_all_sucess       = _(u'All images added with success.'),
            ul_msg_some_sucess      = _(u' images added with success, '),
            ul_msg_some_errors      = _(u" uploads return an error."),
            ul_msg_failed           = _(u"Failed"),
            ul_error_try_again      = _(u"please try again."),
            ul_error_empty_file     = _(u"This image is empty:"),
            ul_error_file_large     = _(u"This image is too large:"),
            ul_error_maxsize_is     = _(u"maximum file size is:"),
            ul_error_bad_ext        = _(u"This file has an invalid extension:"),
            ul_error_onlyallowed    = _(u"Allowed types are:"),
            ul_error_already_exists = _(u"This image already exists with the same name on server:"),
            ul_error_server         = _(u"An server error occurred."),
        )
        
        return XHR_UPLOAD_JS % strings

class AddImage(BrowserView):
    """ Upload an image
    """

    def _utranslate(self, msg):
        context = aq_inner(self.context)
        return context.translate(msg, domain="xhostplus.gallery")

    def __init__(self, context, request):
        self.context = context
        self.request = request

        cookie = self.request.form.get("cookie")
        if cookie:
            self.request.cookies["__ac"] = decode(cookie)

    def __call__(self):
        _ = self._utranslate
        if self.request.HTTP_X_REQUESTED_WITH:
            # using ajax upload
            file_name = urllib.unquote(self.request.HTTP_X_FILE_NAME)
            try :
                bodyfile = self.request.BODYFILE
                file_data = bodyfile.read()
                bodyfile.seek(0)
            except :
                return json.dumps({u'error': _(u"An unknown server error occurred")})

        content_type = mimetypes.guess_type(file_name)[0] or ""

        # call all file mutator utilities
        for mutator in component.getAllUtilitiesRegisteredFor(IFileMutator):
            file_name, file_data, content_type = mutator(file_name, file_data, content_type)

        if file_data:
            factory = IFileFactory(self.context)
            try:
                f = factory(file_name, content_type, file_data)
            except:
                return json.dumps({u'error': _(u"The file %(file)s already exists") % {'file' : file_name}})

            return json.dumps({u'success': True})
        return json.dumps({u'error': _(u"The file %(file)s is empty") % {'file' : file_name}})


from random import sample

from zope.interface import Interface
from zope.interface import implements

from plone.app.portlets.portlets import base
from plone.portlets.interfaces import IPortletDataProvider

from zope import schema
from zope.formlib import form
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary
from zope.app.schema.vocabulary import IVocabularyFactory
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

from xhostplus.gallery import galleryMessageFactory as _

class IGalleryPortlet(IPortletDataProvider):
    """A portlet

    It inherits from IPortletDataProvider because for this portlet, the
    data that is being rendered and the portlet assignment itself are the
    same.
    """

    path = schema.Choice(title=_(u'Gallery path'),
                       description=_(u'The path to the gallery.'),
                       required=True,
                       vocabulary='xhostplus.gallery.GalleryVocabulary')

    count = schema.Int(title=_(u'Number of images'),
                       description=_(u'How many images to list.'),
                       required=True,
                       default=5)


class Assignment(base.Assignment):
    """Portlet assignment.

    This is what is actually managed through the portlets UI and associated
    with columns.
    """

    implements(IGalleryPortlet)

    def __init__(self, path='/', count=5):
        self.count = count
        self.path = path

    @property
    def title(self):
        """This property is used to give the title of the portlet in the
        "manage portlets" screen.
        """
        return "Gallery Portlet"

    def getGallery(self, context):
        """Returns the selected gallery object
        """
        catalog = getToolByName(context, 'portal_catalog') # getting the catalog for searching objects
        result = catalog(portal_type='Image Gallery',
                         review_state='published',
                         path={
                             'query': self.path, 
                             'depth': 0,
                         })

        if len(result) == 1:
            return result[0].getObject()
        return None

    def getImages(self, context):
        """Returns the selected gallery object
        """
        catalog = getToolByName(context, 'portal_catalog') # getting the catalog for searching objects
        results = catalog(portal_type='Image',
                         path={
                             'query': self.path, 
                             'depth': 1,
                         })

        count = self.count
        if count > len(results):
            count = len(results)

        return sample([result.getObject() for result in results], count)

class Renderer(base.Renderer):
    """Portlet renderer.

    This is registered in configure.zcml. The referenced page template is
    rendered, and the implicit variable 'view' will refer to an instance
    of this class. Other methods can be added and referenced in the template.
    """

    render = ViewPageTemplateFile('galleryportlet.pt')

    @property
    def available(self):
        return self.gallery() and len(self.images()) > 0

    def gallery(self):
        return self.data.getGallery(self.context)

    def images(self):
        return self.data.getImages(self.context)

class AddForm(base.AddForm):
    """Portlet add form.

    This is registered in configure.zcml. The form_fields variable tells
    zope.formlib which fields to display. The create() method actually
    constructs the assignment that is being added.
    """
    form_fields = form.Fields(IGalleryPortlet)

    def create(self, data):
        return Assignment(**data)


class EditForm(base.EditForm):
    """Portlet edit form.

    This is registered with configure.zcml. The form_fields variable tells
    zope.formlib which fields to display.
    """
    form_fields = form.Fields(IGalleryPortlet)

class GalleryVocabulary(object):
    """ Ad Unit sizes """

    implements(IVocabularyFactory)

    def __call__(self, context):
        catalog = getToolByName(context, 'portal_catalog') # getting the catalog for searching objects
        results = catalog(portal_type='Image Gallery', review_state='published')
        found = []

        for result in results:
            found.append(SimpleTerm(result.getPath(), result.getPath(), result.getPath()))

        return SimpleVocabulary(found)


GalleryVocabularyFactory = GalleryVocabulary()



from zope.interface import implements, Interface

from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName

from xhostplus.gallery import galleryMessageFactory as _


class IGalleryView(Interface):
    """
    Gallery view interface
    """


class GalleryView(BrowserView):
    """
    Gallery browser view
    """
    implements(IGalleryView)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def portal_catalog(self):
        return getToolByName(self.context, 'portal_catalog')

    @property
    def portal(self):
        return getToolByName(self.context, 'portal_url').getPortalObject()



from zope import interface

class IFileMutator(interface.Interface):
    """ a file mutator utility

        returns -> (file_name, file_data, content_type)
    """

class IUploadingCapable(interface.Interface):
    """ Any container/object that is supported for uploading into.
    """


"""Definition of the Image Gallery content type
"""

from zope.interface import implements, directlyProvides

from Products.Archetypes import atapi
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata

from xhostplus.gallery import galleryMessageFactory as _
from xhostplus.gallery.interfaces import IImageGallery
from xhostplus.gallery.config import PROJECTNAME

ImageGallerySchema = folder.ATFolderSchema.copy() + atapi.Schema((

    # -*- Your Archetypes field definitions here ... -*-

    atapi.BooleanField('slideshow',
        languageIndependent=1,
        widget=atapi.BooleanWidget(
              label=_(u'Enable slideshow'),
              description=_(u"Whether the slideshow is enabled or not."),
        ),
        required=False,
        searchable=False,
        default=False,
    ),

    atapi.IntegerField('slideshow_interval',
        languageIndependent=1,
        widget=atapi.IntegerWidget(
              label=_(u'Slideshow interval'),
              description=_(u"Seconds to wait for switching to the next image."),
        ),
        required=True,
        searchable=False,
        default=5,
    ),

))

# Set storage on fields copied from ATFolderSchema, making sure
# they work well with the python bridge properties.

ImageGallerySchema['title'].storage = atapi.AnnotationStorage()
ImageGallerySchema['description'].storage = atapi.AnnotationStorage()

schemata.finalizeATCTSchema(
    ImageGallerySchema,
    folderish=True,
    moveDiscussion=False
)

class ImageGallery(folder.ATFolder):
    """An image gallery folder"""
    implements(IImageGallery)

    meta_type = "Image Gallery"
    schema = ImageGallerySchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')
    
    # -*- Your ATSchema to Python Property Bridges Here ... -*-

    def validate_slideshow_interval(self, value):
        try:
            value = int(value)
        except ValueError:
            return _(u"The interval time should be an integer.")
        if value < 1:
            return _(u"The interval time should not be lower than 1.")
        if value > 30:
            return _(u"The interval time should not be greater than 30.")
        return None

atapi.registerType(ImageGallery, PROJECTNAME)



from zope import schema
from zope.interface import Interface

from zope.app.container.constraints import contains
from zope.app.container.constraints import containers

from xhostplus.gallery import galleryMessageFactory as _

class IImageGallery(Interface):
    """An image gallery folder"""
    
    # -*- schema definition goes here -*-



import transaction
from thread import allocate_lock

from zope import interface
from zope import component

from zope.event import notify
from zope.lifecycleevent import ObjectModifiedEvent
from zope.filerepresentation.interfaces import IFileFactory
from zope.app.container.interfaces import INameChooser
from plone.i18n.normalizer.interfaces import IIDNormalizer

from Products.Archetypes.event import ObjectInitializedEvent
from Products.CMFPlone import utils as ploneutils
from Products.CMFCore import utils as cmfutils

from interfaces import IUploadingCapable

upload_lock = allocate_lock()

class UploadingCapableImageFactory(object):
    interface.implements(IFileFactory)
    component.adapts(IUploadingCapable)

    def __init__(self, context):
        self.context = context

    def __call__(self, name, content_type, data):
        ctr = cmfutils.getToolByName(self.context, 'content_type_registry')
        type_ = ctr.findTypeName(name.lower(), '', '') or 'File'

        # XXX: quick fix for german umlauts
        name = name.decode("utf8")

        normalizer = component.getUtility(IIDNormalizer)
        chooser = INameChooser(self.context)

        # otherwise I get ZPublisher.Conflict ConflictErrors
        # when uploading multiple files
        upload_lock.acquire()

        # this should fix #8
        newid = chooser.chooseName(normalizer.normalize(name), self.context.aq_parent)
        try:
            transaction.begin()
            obj = ploneutils._createObjectByType(type_, self.context, newid)
            mutator = obj.getPrimaryField().getMutator(obj)
            mutator(data, content_type=content_type)
            obj.setTitle(name)
            obj.reindexObject()

            notify(ObjectInitializedEvent(obj))
            notify(ObjectModifiedEvent(obj))

            transaction.commit()
        finally:
            upload_lock.release()
        return obj


# -*- coding: utf-8 -*-
"""
This module contains the tool of xhostplus.gallery
"""
import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '1.6.0'

long_description = (
    read('README.txt')
    + '\n' +
    read('CHANGES.txt')
    )

tests_require=['zope.testing']

setup(name='xhostplus.gallery',
      version=version,
      description="XHOSTPLUS Gallery",
      long_description=long_description,
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Framework :: Plone',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        ],
      keywords='Gallery Plugin Product Plone XHOSTPLUS',
      author='Andreas Stocker',
      author_email='andreas.stocker@xhostplus.at',
      url='http://www.xhostplus.at',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['xhostplus', ],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'setuptools',
        # -*- Extra requirements: -*-
      ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      test_suite = 'xhostplus.gallery.tests.test_docs.test_suite',
      entry_points="""
      # -*- entry_points -*-
      [distutils.setup_keywords]
      paster_plugins = setuptools.dist:assert_string_list

      [egg_info.writers]
      paster_plugins.txt = setuptools.command.egg_info:write_arg
      """,
      paster_plugins = ["ZopeSkel"],
      )



from zope.component import getUtility, getMultiAdapter

from plone.portlets.interfaces import IPortletType
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletAssignment
from plone.portlets.interfaces import IPortletDataProvider
from plone.portlets.interfaces import IPortletRenderer

from plone.app.portlets.storage import PortletAssignmentMapping

from xhostplus.gallery.portlets import galleryportlet
from xhostplus.gallery.portlets.tests.base_galleryportlet import TestCase


class TestPortlet(TestCase):

    def afterSetUp(self):
        self.setRoles(('Manager',))

    def test_portlet_type_registered(self):
        portlet = getUtility(IPortletType, name='xhostplus.gallery.portlets.GalleryPortlet')
        self.assertEquals(portlet.addview, 'xhostplus.gallery.portlets.GalleryPortlet')

    def test_interfaces(self):
        # TODO: Pass any keywoard arguments to the Assignment constructor
        portlet = galleryportlet.Assignment()
        self.failUnless(IPortletAssignment.providedBy(portlet))
        self.failUnless(IPortletDataProvider.providedBy(portlet.data))

    def test_invoke_add_view(self):
        portlet = getUtility(IPortletType, name='xhostplus.gallery.portlets.GalleryPortlet')
        mapping = self.portal.restrictedTraverse('++contextportlets++plone.leftcolumn')
        for m in mapping.keys():
            del mapping[m]
        addview = mapping.restrictedTraverse('+/' + portlet.addview)

        # TODO: Pass a dictionary containing dummy form inputs from the add form
        addview.createAndAdd(data={})

        self.assertEquals(len(mapping), 1)
        self.failUnless(isinstance(mapping.values()[0], galleryportlet.Assignment))

    # NOTE: This test can be removed if the portlet has no edit form
    def test_invoke_edit_view(self):
        mapping = PortletAssignmentMapping()
        request = self.folder.REQUEST

        mapping['foo'] = galleryportlet.Assignment()
        editview = getMultiAdapter((mapping['foo'], request), name='edit')
        self.failUnless(isinstance(editview, galleryportlet.EditForm))

    def test_obtain_renderer(self):
        context = self.folder
        request = self.folder.REQUEST
        view = self.folder.restrictedTraverse('@@plone')
        manager = getUtility(IPortletManager, name='plone.rightcolumn', context=self.portal)

        # TODO: Pass any keywoard arguments to the Assignment constructor
        assignment = galleryportlet.Assignment()

        renderer = getMultiAdapter((context, request, view, manager, assignment), IPortletRenderer)
        self.failUnless(isinstance(renderer, galleryportlet.Renderer))


class TestRenderer(TestCase):

    def afterSetUp(self):
        self.setRoles(('Manager',))

    def renderer(self, context=None, request=None, view=None, manager=None, assignment=None):
        context = context or self.folder
        request = request or self.folder.REQUEST
        view = view or self.folder.restrictedTraverse('@@plone')
        manager = manager or getUtility(IPortletManager, name='plone.rightcolumn', context=self.portal)

        # TODO: Pass any default keywoard arguments to the Assignment constructor
        assignment = assignment or galleryportlet.Assignment()
        return getMultiAdapter((context, request, view, manager, assignment), IPortletRenderer)

    def test_render(self):
        # TODO: Pass any keywoard arguments to the Assignment constructor
        r = self.renderer(context=self.portal, assignment=galleryportlet.Assignment())
        r = r.__of__(self.folder)
        r.update()
        output = r.render()
        # TODO: Test output


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestPortlet))
    suite.addTest(makeSuite(TestRenderer))
    return suite
